/*
Copyright Renaud GICQUEL 2013
renaud.gicquel@s4e2.com

Ce logiciel est un programme informatique servant  modliser des systmes nergtiques.   

Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA  sur le site "http://www.cecill.info".  

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons, seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et les concdants successifs.  

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,   la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant  donn sa spcificit de logiciel libre, qui peut le rendre complexe   manipuler et qui le rserve donc  des dveloppeurs et des professionnels avertis possdant  des  connaissances  informatiques approfondies.  Les utilisateurs sont donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,   l'utiliser et l'exploiter dans les mmes conditions de scurit.   

Le fait que vous puissiez accder  cet en-tte signifie que vous avez  pris connaissance de la licence CeCILL, et que vous en avez accept les termes. 

This software is a computer program whose purpose is to model energy systems.  

This software is governed by the CeCILL  license under French law and abiding by the rules of distribution of free software.  You can  use,  modify and/ or redistribute the software under the terms of the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".   

As a counterpart to the access to the source code and  rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty  and the software's author,  the holder of the economic rights,  and the successive licensors  have only  limited liability.   

In this respect, the user's attention is drawn to the risks associated with loading,  using,  modifying and/or developing or reproducing the software by the user in light of its specific status of free software, that may mean  that it is complicated to manipulate,  and  that  also therefore means  that it is reserved for developers  and  experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the software's suitability as regards their requirements in conditions enabling the security of their systems and/or  data to be ensured and,  more generally, to use and operate it in the  same conditions as regards security.   

The fact that you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.  
 * 
 * @author Renaud GICQUEL
*/
package extThopt;

import java.util.*; 
import java.io.*;

import javax.swing.*;

/**
* This class enables you to add substances to Thermoptim's data base
* To add one substance, subclass ExtSubstance by providing the appropriate methods
* in particular, getType() must return the compressor code as it will appear in
* the substance list
*<p>This class should not be modified
 * 
 * @author Renaud GICQUEL
*/
public class ExtSubstance 
{
    String type;
    String chemForm="";
    double M, PC,  TC,  VC, Tmini,  Tmaxi,  Pmini,  Pmaxi;
    int typeCorps=5;
    
    public ExtSubstance (){
        super();
    }
  
    public boolean isMolarFraction(){
        return true;
    }
    
    //identification de la classe
        
    /**
    * dfinit l'identifiant de la classe
    * <p>
    * defines the class identifier
    * <p>
    * must be overridden by subclasses
    */
    public String getType(){
        return getCompType();
    }
    public String getCompType(){
        return "external substance";
    }

    /**
    * dfinit la description de la classe apparaissant dans le visualisateur
    * <p>
    * defines the class description shown in the viewer
    * <p>
    * should be overridden by subclasses
    */
	public String getClassDescription(){
	    return "external substance class\nbasic generic class";
	}

	
		
	//Variables et mthodes introduites pour les mlanges externes

    /**
    * titre d'un corps pur, ou composition  l'quilibre
    * <p>
    * quality of a pure substance, or composition at equilibrium
    * <p>
    * should be overridden by external mixture classes
	* set equal to 1 by default
    */
    public double[] X={1};
    double x, massMol;
    Vector vMixtures;
    String selectedSyst,selectedFile;
    public double fract_mol[];
    public double fract_mass[];
    StringBuffer nomfich;
	String dir_data=null;
	public File fich;
	public Hashtable mel_externes;
    String[] components;
    double P0,T0,Pref;
    double hv,sv,xx;
    public double[]res=new double[20];
    double[]args=new double[3];
	boolean testInit=false;
	String mixtureFileName;
    public double molarWeights[];
    public double Tmin[];
    public double Tmax[];
    public double Pmax[];
    public double fractType=1;//vaut 1 si molaire, 0 autrement
    int[]paramSPT=new int[2];//passage de paramtres d'initialisation et retour d'erreur
	double[]initProps=new double[30];//proprits d'initialisation du SPT (Mi,PCi...
	String directory="";
    /**
    * renvoie le titre d'un corps pur, ou bien la composition  l'quilibre
    * <p>
    * gets the quality of a pure substance, or the composition at equilibrium
	*<p>
	* not to be overriden by ExtSubstance subclasses
    * @return double[]
    */
    public double[] getQuality(){
        return X;
    }    

    /** returns the molar weights and other initialization values
    */
    public double[] getMolarWeights(){
        return molarWeights;
    }    
    
	/**
	    * renvoie la composition molaire des phases liquide et vapeur pour T et V donns
	    * <p>
	    * gets the molar composition of liquid and vapor phases, T and P_bar being given
		*<p>
		* @param ksat (0 = liquid, 1 = vapor)
	    * @return double[]
	    */
    public double[] getSatComp(double T, double P, int ksat){
        double[]res=new double[1];
        res[0]=ksat;
        return res;
    }
    
    
    public Vector getMixtures()throws MixtureException{
		vMixtures= new Vector();
    	nomfich= new StringBuffer(mixtureFileName); 
	    dir_data=System.getProperty("user.dir")+File.separator+"mixtures"+File.separator;
		try{
			fich = new File(dir_data,nomfich.toString());
			lect_data(fich);
  		}
  		catch(Exception e){//en cas de problme, on gnre une exception pour le fichier error.txt
  		    e.printStackTrace();
  			throw new MixtureException(getType());
  		}
      return vMixtures;
    }
    
    /**
    * modifie la composition d'un mlange externe
    * <p>
    * modifies the composition of an external mixture
    * <p>
    * must be overridden by external mixture classes
    * @param systType String (system type)
    * @param fract_mol double[] (molar fractions)
    */
       public void updateComp(String systType, double fractType, double[] fractmol){

            selectedSyst=systType;
            //rcupration des informations sur le systme considr par la Hashtable
	        Vector liste_comp = (Vector)(mel_externes.get(selectedSyst));
	        
            selectedFile=(String)liste_comp.elementAt(0);//fichier de mlange du systme (XXX.mel)
            int nbComp=Util.lit_i((String)liste_comp.elementAt(1));//nombre de composants du systme
            fract_mol = fractmol;
            components=new String[nbComp];
	        for(int i=0;i<fract_mol.length;i++){
	           components[i]=(String)liste_comp.elementAt(i+2);//noms des composants
	        }
	 /*       //coordonnes critiques (pour les corps purs)
	        PC=Util.lit_d((String)liste_comp.elementAt(nbComp+2));
	        TC=Util.lit_d((String)liste_comp.elementAt(nbComp+3));
	        //intervalle de dfinition des modles utiliss
	        Tmaxi=Util.lit_d((String)liste_comp.elementAt(nbComp+4));
	        Tmini=Util.lit_d((String)liste_comp.elementAt(nbComp+5));
	        //Pression et temprature des enthalpie, entropie et nergie interne de rfrence
	        P0=Util.lit_d((String)liste_comp.elementAt(nbComp+6));
	        T0=Util.lit_d((String)liste_comp.elementAt(nbComp+7));
	        */
    }
/**
	 * Lecture du fichier de donnes dfinissant les
	 * gaz composs
	 * 
	 * Reads the data file of the compound gases
	 * 
	 * @param fich File
	 * @param prot "true" if protected gas file
	 */
	 public void lect_data(File fich) throws Exception{
	    FileReader flux_fich=null;
	    BufferedReader flux_tamp=null;
		try{
			flux_fich = new FileReader(fich);
			flux_tamp = new BufferedReader(flux_fich);
			String ligne_data;

			ligne_data = flux_tamp.readLine();//premire ligne de commentaires
			StringTokenizer st = new StringTokenizer(ligne_data,"\t");

			//lignes de donnes
			StringBuffer test=new StringBuffer("");
			String compose="";
		    while (((ligne_data = flux_tamp.readLine())!= null)&& (!(test.toString().equals("Fin")))){
				//nom du gaz compos
				st = new StringTokenizer(ligne_data,"\t");
				String mixtureName = st.nextToken();
				String mixtureFile="";
				if(st.hasMoreTokens())mixtureFile= st.nextToken();
				int nbComp=0;
				if(st.hasMoreTokens())nbComp=Util.lit_i(st.nextToken());
				components=new String[nbComp] ;

				test.setLength(0);
				test.append(mixtureName);
 	            Object[]obj1=new Object[2];
				obj1[0]=mixtureName;
		        Vector liste_composes = new Vector();
 				liste_composes.addElement(mixtureFile);
 				liste_composes.addElement(Util.aff_i(nbComp));
				int j_dat=-1;
				if(st.hasMoreTokens()){
					for(int k=0;k<nbComp;k++){
						j_dat++;
						compose = st.nextToken();
						components[j_dat]=compose;
						liste_composes.addElement(compose);
			        }
			        //J'ai supprim ces initialisations inutiles, mais pas compltement 
			        //enlev les rfrences  ces variables, qu'il faut donc continuer  fournir
			        M=44;//Util.lit_d(Util.extr_value(ligne_data,"MolMass"));
			        PC=100;//Util.lit_d(Util.extr_value(ligne_data,"PC"));
			        liste_composes.addElement(Util.aff_d(PC));
			        TC=300;//Util.lit_d(Util.extr_value(ligne_data,"TC"));
			        liste_composes.addElement(Util.aff_d(TC));
			        Tmaxi=500;//Util.lit_d(Util.extr_value(ligne_data,"Tmaxi"));
			        liste_composes.addElement(Util.aff_d(Tmaxi));
			        Tmini=200;//Util.lit_d(Util.extr_value(ligne_data,"Tmini"));
			        liste_composes.addElement(Util.aff_d(Tmini));
			        P0=1;//Util.lit_d(Util.extr_value(ligne_data,"P0"));
			        liste_composes.addElement(Util.aff_d(P0));
			        T0=300;//Util.lit_d(Util.extr_value(ligne_data,"T0"));
			        liste_composes.addElement(Util.aff_d(T0));
				
				}
				if(!mixtureName.equals("Fin")){
                   	mel_externes.put(mixtureName,liste_composes);//chargement dans la HashTable des rfrences du modle
				    obj1[1]=components;
	   				vMixtures.addElement(obj1);
	   			}
		    }
			flux_tamp.close();
			flux_fich.close();
	
		} 
		catch(FileNotFoundException e) {
		    String message="Le_fichier "+fich.getAbsolutePath()+" est introuvable\n Le mlange externe ne peut tre cr";  
		    JOptionPane.showMessageDialog(new JFrame(),message);
		    e.printStackTrace();
		    throw new Exception();
		}
		catch(IOException e) {
		    String message="Le_fichier "+fich.getAbsolutePath()+" est mal configur\n Le mlange externe ne peut tre cr";  
		    JOptionPane.showMessageDialog(new JFrame(),message);
		    e.printStackTrace();
		    throw new Exception();
		}
	}
	
	//Mthodes de calcul gnrales pour les corps externes
 
    /**
    * calcule l'tat complet du corps pour un mlange externe
    * <p>
    * calculates the substance state for an external mixture
	*<p>
	* must be overriden by external mixtures
    * @param T double (temperature)
    * @param P double (pressure)
    * @param x double (quality)
    */
    public double[] CalcPropCorps (double T, double p, double x) {
        double[]val=new double[6];
        return val;
    }
     
    /**
    * calcule la pression P, h et T tant donns
    * <p>
    * calculates the pressure P, h and T being known
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $hv double (enthalpy)
    * @param $T double (temperature)
    */
	public double getP_from_hT(double $hv,double $T){return 0;}
    /**
    * calcule la temprature T, h et P tant donns
    * <p>
    * calculates the temperature T, h and P being known
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $hv double (enthalpy)
    * @param $P double (pressure)
    */
	public double getT_from_hP(double $hv,double $P){return 0;}
    /**
    * calcule la pression P, s et T tant donns
    * <p>
    * calculates the pressure P, s and T being known
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $sv double (entropy)
    * @param $T double (temperature)
    */
	public double getP_from_sT(double $sv,double $T){return 0;}
    /**
    * calcule la pression P, s et T tant donns
    * <p>
    * calculates the pressure P, s and T being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $sv double (entropy)
    * @param $T double (temperature)
    * @param pmin double (minimum pressure)
    * @param pmax double (maximum pressure)
    */
	public double getP_from_sT(double $sv,double $T, double pmin, double pmax){
    	    return 0.;
	}
  
    /**
    * calcule la temprature T, s et P tant donns
    * <p>
    * calculates the temperature T, s and P being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $sv double (entropy)
    * @param $P double (pressure)
    */
	public double getT_from_sP(double $sv,double $p){
	    return 0.;
	}

    /**
    * calcule la temprature T, s et v tant donns
    * <p>
    * calculates the temperature T, s and v being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $sv double (entropy)
    * @param $v double (volume)
    */
	public double getT_from_sv(double $s,double $v){
	    return 0.;
	}
    
    /**
    * calcule la temprature T, u et v tant donns
    * <p>
    * calculates the temperature T, u and v being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $u double (internal energy)
    * @param $v double (volume)
    */
	public double getT_from_uv(double $u,double $v){
	    return 0.;
	}
    
    /**
    * calcule la pression P, v et T tant donns
    * <p>
    * calculates the pressure P, v and T being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $v double (volume)
    * @param $T double (temperature)
    */
	public double getP_from_vT(double $v,double $T){
	    return 0.;
	}
    /**
    * calcule P, T et x, s et h tant donns
    * <p>
    * calculates P, T and x, s et h being known
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $v double (volume)
    * @param $T double (temperature)
    */
    public double[] getPTx_from_sh(double s, double h){
        double[]val=new double[3];
        return val;
    }


    /**
    * calcule la pression de saturation
    * <p>
    * calculates the saturation pressure
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    * @param x double (quality)
    */
	public double getSatPressure(double T, double x){
	    return 0.;
	}
	
	

    /**
    * calcule la temprature de saturation
    * <p>
    * calculates the saturation temperature
	* <p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param P double (pressure)
    * @param x double (quality)
    */
	public double getSatTemperature(double P, double x){
	    return 0.;
	}
    
    /**
    * calcule l'enthalpie
    * <p>
    * calculates the enthalpy
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcH (double T, double P,double x) {return 0;}

    /**
    * calcule la capacit thermique massique  volume constant
    * <p>
    * calculates the constant volume mass thermal capacity
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcCv (double T, double P,double x) {return 0.;}

    /**
    * calcule la capacit thermique massique  pression constante
    * <p>
    * calculates the constant pressure mass thermal capacity
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcCp (double T, double P,double x) {return 0;}
	
    /**
    * calcule Cp
    * <p>
    * calculates Cp
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param Cv double (constant volume mass thermal capacity)
    */
    public double getCp(double Cv) {
        return Cv;
    }

    /**
    * calcule le volume
    * <p>
    * calculates the volume
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcV (double T, double P,double x) {return 0;}
  
    /**
    * calcule l'entropie
    * <p>
    * calculates the entropy
	*<p>
	* to be overriden ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcS (double T, double P,double x) {return 0;}
  
    /**
    * calcule le titre
    * <p>
    * calculates the quality
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param T double (temperature)
    */
	public double calcXx (double T, double P,double x) {return 0;}
  
    /**
    * cette mthode calcule la viscosit dynamique liquide en Pa.s
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $T double (temprature)
    */
    public double getLiquidViscosity(double $T) {return 0;}
    public double getLiquidViscosity(double $T, double P) {return getLiquidViscosity($T);}

    /**
    * cette mthode calcule la viscosit dynamique vapeur en Pa.s
 	*<p>
	* to be overriden by ExtSubstance subclasses
   * @return double
    * @param $T double (temprature)
    */
    public double getVaporViscosity(double $T) {return 0;}
    public double getVaporViscosity(double $T, double P) {return getVaporViscosity($T);}

    /**
    * cette mthode calcule la conductivit thermique liquide en W/m/K
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $T double (temprature)
    */
    public double getLiquidConductivity(double $T) {return 0;}
    public double getLiquidConductivity(double $T, double P) {return getLiquidConductivity($T);}

    /**
    * cette mthode calcule la conductivit thermique vapeur en W/m/K
	*<p>
	* to be overriden by ExtSubstance subclasses
    * @return double
    * @param $T double (temprature)
    */
    public double getVaporConductivity(double $T) {return 0;}  
    public double getVaporConductivity(double $T, double P) {return getVaporConductivity($T);}  
    
	
    public double[] getTransportProperties(double T, double rho){
        double[]res={0,0};
	    if(xx==0){
	        res[0]=getLiquidViscosity(T, rho);
	        res[1]=getLiquidConductivity(T, rho);
	    }
	    else{
	        res[0]=getVaporViscosity(T, rho);
	        res[1]=getVaporConductivity(T, rho);
	    }
    	return res;
    }
    
    /**
     * cette mthode renvoie les proprits critiques 
 	*<p>
 	* to be overriden in ExtSubstance mixture subclasses
     * @return double[]
     */
   public double[] getCriticalParameters(){
        double[]res={0,0,0,0};
    	return res;
    }
   
   public  void setSubstanceParameters(double[] params){ }
/*	    double[]props=new double[4];
    	props[0] = TC;
    	props[1] = PC;
    	props[2] = VC;
    	props[3] = M;
    	return props;*/

}